﻿/******************************************************************************************************************************************************************
Class Name:  DepartmentsView
Purpose: 
DepartmentsView is the code behind for Departments View (MPT-002)
******************************************************************************************************************************************************************/
storeTies.DepartmentsView = function (masterDataService, spinner) {
    var
       _$departmentsDialogDiv,
       _$departmentsGrid,
       _$departmentNumberTextBox,
       _$departmentNameTextBox,
       _$departmentsSelectedCountTextBox,
       _departmentListInvoker,
       _departmentsToSelect,
       _gridElement,

       init = function () {
           _$departmentsDialogDiv = $("#departmentsDialogDiv");
           _$departmentsGrid = $("#departmentsGrid");
           _$departmentNumberTextBox = $("#departmentNumberTextBox"),
           _$departmentNameTextBox = $("#departmentNameTextBox"),
           _$departmentsSelectedCountTextBox = $("#departmentsSelectedCountTextBox");
           _departmentListInvoker = "";
           _departmentsToSelect = [];

           _intializeDepartmentsDialog();

           $(document).on("openDepartmentsListPopup", function (e, options) {

               var gridOptions = {
                   columnSettings: {
                       columnKey: "Id",
                       allowSorting: true,
                       firstSortDirection: "ascending"
                   }
               };

               _intializeDepartmentsGrid(gridOptions);
               _checkPreSelectedDepartments(options.departmentsSelected);
               _setSelectedRowsCount();

               _departmentListInvoker = options.sender;
               _departmentsToSelect = options.departmentsSelected;

               _$departmentsDialogDiv.igDialog("open");


           });

           $("#departmentNumberTextBox").on("keyup", function () {
               var matchedDepartmentName;
               _$departmentNameTextBox.val("");
               setTimeout(function () {
                   matchedDepartmentName = _searchGrid(_$departmentNumberTextBox.val(), "Id", 1, _matchForNumber, "Name");
                   if (matchedDepartmentName !== null) {
                       _$departmentNameTextBox.val(matchedDepartmentName);
                   }
               }, 500);
           });

           $("#departmentNameTextBox").on("keyup", function () {
               var matchedDepartmentNumber, departmentNameText = _$departmentNameTextBox.val();
               _$departmentNumberTextBox.val("");
               if (departmentNameText !== "") {
                   if (departmentNameText.length >= 1)
                       setTimeout(function () {
                           matchedDepartmentNumber = _searchGrid(_$departmentNameTextBox.val(), "Name", 2, _matchForStartingString, "Id");
                           if (matchedDepartmentNumber)
                           {
                               _$departmentNumberTextBox.val(matchedDepartmentNumber);
                           }
                       }, 500);
               }
           });

           $("#okButtonOnDepartmentsDialog").on("click", function () {
               var curentSelectedDepartmentIds;
               curentSelectedDepartmentIds = _getCurrentlySelectedDepartmentIds();
               _triggerDepartmentsSelectionDoneEventAndCloseDialog(curentSelectedDepartmentIds);
               _clearSelectionsAndFilterTexts();
               _departmentListInvoker = "";
           });

           $("#cancelButtonOnDepartmentsDialog").on("click", function () {
               _$departmentsDialogDiv.igDialog("close");
               _clearSelectionsAndFilterTexts();
           });
       },

       _intializeDepartmentsDialog = function () {
           _$departmentsDialogDiv.igDialog({
               headerText: "MPT-002 Departments",
               state: "closed",
               modal: true,
               resizable: false,
               minHeight: 600,
               minWidth: 600,
               stateChanged: function (evt, ui) {
                   if (ui.action === "close") {
                       _clearSelectionsAndFilterTexts();
                   }
               }
           });
       },

       _intializeDepartmentsGrid = function (gridOptions) {

           var curentSelectedDepartmentIds = [];

           _$departmentsGrid.igGrid({
              height: "380px",
               width: "520px",
               dataSource: allDeparments,
               autoGenerateColumns: false,
               alternateRowStyles: false,
               virtualization: true,
               virtualizationMode: "continuous",
               primaryKey: "Id",
               columns: [
                   { headerText: "Department", key: "Id", dataType: "int", width: "120px" },
                   { headerText: "Name", key: "Name", dataType: "string", width: "140px" }
               ],
               features: [
                   {
                       name: "RowSelectors",
                       enableCheckBoxes: true,
                       enableRowNumbering: false,
                       checkBoxStateChanged: function (evt, ui) {
                           _setSelectedRowsCount();
                       },
                       checkBoxStateChanging: function (evt, ui) {
                           if (ui.isHeader) {
                               if (ui.currentState === "off") {
                                   _$departmentsGrid.igGridSelection("clearSelection");
                               }
                           }
                           return true;
                       }
                   },
                   {
                       name: "Sorting",
                       applySortedColumnCss: false,
                       columnSettings: [gridOptions.columnSettings],
                       columnSorting: function (evt, ui) {
                           curentSelectedDepartmentIds = _getCurrentlySelectedDepartmentIds();
                           _$departmentsGrid.igGrid("destroy");
                           _intializeDepartmentsGrid({
                               columnSettings: {
                                   columnKey: ui.columnKey,
                                   allowSorting: true,
                                   currentSortDirection: ui.direction
                               }
                           });
                           _loadAndBindData()
                                .done(function () {
                                    _checkPreSelectedDepartments(curentSelectedDepartmentIds);
                                    _setSelectedRowsCount();
                                });
                           return false;
                       }
                   },
                   {
                       name: "Selection",
                       multipleSelection: true,
                       rowSelectionChanged: function (evt, ui) {
                           _setSelectedRowsCount();
                       }
                   },
                    {
                        name: "Resizing"
                    },
                    {
                        name : "Resizing",
                        columnResized: function (evt, ui) {
                            var tds = ui.owner.element.find("tr td");
                            
                            $.each(tds, function (index, td) {
                                var $td = $(td);
                                $td.attr("height", 20);
                            });
                        }
                    }

               ],
               rowsRendered: function (evt, ui) {
                                    ui.owner.element.find("tr td:nth-child(2)").css("text-align", "center");
               }
           });
       },


       _checkPreSelectedDepartments = function (departmentIds) {
           var dataView = _$departmentsGrid.data('igGrid').dataSource.dataView();
           var matchedRowIndexes = [];
           if (departmentIds.length > 0) {
               $.each(dataView, function (rowIndex, dataRow) {
                   $.each(departmentIds, function (index, departmentId) {
                       if (dataRow.Id == departmentId) {
                           _$departmentsGrid.igGridSelection("selectRow", rowIndex);
                           matchedRowIndexes.push(rowIndex);
                       }
                   });
               });
           }
       },

       _setSelectedRowsCount = function () {
           var selectedRowsCount = _$departmentsGrid.igGrid("selectedRows").length;
           _$departmentsSelectedCountTextBox.val(selectedRowsCount);
       },

       _searchGrid = function (searchText, columnName, columnIndex, matchFunc, columnToReturn) {
           var dataView,
               rowFound,
               columnValueToReturn;

           rowFound = false;
           dataView = _$departmentsGrid.data('igGrid').dataSource.dataView();
           if (searchText !== "") {
               $.each(dataView, function (rowIndex, dataRow) {
                   if (matchFunc(dataRow[columnName], searchText) && rowFound === false) {
                       _$departmentsGrid.igGrid("virtualScrollTo", rowIndex);
                       if (dataRow[columnName].toString().trim().toLowerCase() === searchText.trim().toLowerCase()) {
                           columnValueToReturn = dataRow[columnToReturn].toString().trim();
                       }
                       _$departmentsGrid.igGridSelection("selectCell", columnIndex, 1);
                       rowFound = true;
                   }
               });
           }
           return columnValueToReturn;
       },

       _matchForNumber = function (value1, value2) {
           return parseInt(value1) === parseInt(value2);
       },

        _matchForString = function (value1, value2) {
            return value1.trim().toLowerCase() === value2.trim().toLowerCase();
        },

       _matchForStartingString = function (value1, value2) {
           return value1.toLowerCase().indexOf(value2.toLowerCase()) === 0;
       },

       _clearSelectionsAndFilterTexts = function () {
           _$departmentNumberTextBox.val("");
           _$departmentNameTextBox.val("");
           try {
               _$departmentsGrid.igGridSelection("clearSelection");
               _$departmentsGrid.igGrid("destroy");
           }
           catch (error) {
               //alert(error);
           }
       },

       _getCurrentlySelectedDepartmentIds = function () {
           var curentSelectedDepartmentIds,
               selectedDepartments,
               dataView;

           curentSelectedDepartmentIds = [];
           selectedDepartments = _$departmentsGrid.igGridSelection("selectedRows");
           dataView = _$departmentsGrid.data('igGrid').dataSource.dataView();
           $.each(selectedDepartments, function (index, department) {
               curentSelectedDepartmentIds.push(dataView[department.index]["Id"]);
           });
           return curentSelectedDepartmentIds;
       },

       _triggerDepartmentsSelectionDoneEventAndCloseDialog = function (selectedDepartmentIds) {
           var options = { selectedDepartmentIds: selectedDepartmentIds, sender: _departmentListInvoker };
           $(document).trigger("departmentSelectionsDone", [options]);
           _$departmentsDialogDiv.igDialog("close");
       }

    return {
        init: init
    }

}(storeTies.MasterDataService, storeTies.Spinner);


